@@ -1,12 +1,10 @@
 import weakref
-try:
-    from html import escape  # Python 3
-except ImportError:
-    from cgi import escape  # Can't only use this since it was removed in Python 3.8 and up
 from collections import Counter
-from pyqtgraph import PlotItem, ViewBox
+from pyqtgraph import AxisItem, PlotDataItem, PlotItem, ViewBox
+from typing import List
 from .multi_axis_viewbox import MultiAxisViewBox
 from .multi_axis_viewbox_menu import MultiAxisViewBoxMenu
+from ..utilities import is_qt_designer
 
 
 class MultiAxisPlot(PlotItem):
@@ -32,17 +30,21 @@ class MultiAxisPlot(PlotItem):
         super(MultiAxisPlot, self).__init__(viewBox=viewBox, axisItems=axisItems, **kargs)
 
         self.curvesPerAxis = Counter()  # A simple mapping of AxisName to a count of curves that using that axis
+        self.axesOriginalRanges = {}  # Dict from axis name to floats (x, y) representing original range of the axis
 
         # A set containing view boxes which are stacked underneath the top level view. These views will be needed
-        # in order to support multiple axes on the same plot. This set will remain empty if the plot has only one set of axes
+        # in order to support multiple axes on the same plot. This set will remain empty if the plot has only
+        # one set of axes
         self.stackedViews = weakref.WeakSet()
         viewBox.sigResized.connect(self.updateStackedViews)
 
         # Signals that will be emitted when mouse wheel or mouse drag events happen
         self.vb.sigMouseDragged.connect(self.handleMouseDragEvent)
         self.vb.sigMouseWheelZoomed.connect(self.handleWheelEvent)
+        self.vb.setZValue(100)  # Keep this view box on top
         if self.vb.menuEnabled():
-            self.vb.menu.sigMouseModeChanged.connect(self.changeMouseMode)
+            self.connectMenuSignals(self.vb.menu)
+        self.stackedViews.add(self.vb)
 
     def addAxis(self, axis, name, plotDataItem=None, setXLink=False, enableAutoRangeX=True, enableAutoRangeY=True,
                 minRange=-1.0, maxRange=1.0):
@@ -74,17 +76,24 @@ class MultiAxisPlot(PlotItem):
         # Create a new view box to link this axis with
         self.axes[str(name)] = {'item': axis, 'pos': None}  # The None will become an actual position in rebuildLayout() below
         view = MultiAxisViewBox()
+        view.menu = MultiAxisViewBoxMenu(view)
         view.setYRange(minRange, maxRange)
         view.enableAutoRange(axis=ViewBox.XAxis, enable=enableAutoRangeX)
         view.enableAutoRange(axis=ViewBox.YAxis, enable=enableAutoRangeY)
         self.axes['bottom']['item'].linkToView(view)  # Ensure the x axis will update when the view does
 
         view.setMouseMode(self.vb.state['mouseMode'])  # Ensure that mouse behavior is consistent between stacked views
+        self.connectMenuSignals(view.menu)
         axis.linkToView(view)
 
         if plotDataItem is not None:
             self.linkDataToAxis(plotDataItem, name)
 
+        if enableAutoRangeY:
+            self.axesOriginalRanges[name] = (None, None)
+        else:
+            self.axesOriginalRanges[name] = (minRange, maxRange)
+
         self.scene().addItem(view)
         self.addStackedView(view)
 
@@ -109,6 +118,20 @@ class MultiAxisPlot(PlotItem):
         view.sigMouseWheelZoomed.connect(self.handleWheelEvent)
         self.vb.sigHistoryChanged.connect(view.scaleHistory)
 
+    def connectMenuSignals(self, view_box_menu: MultiAxisViewBoxMenu) -> None:
+        """
+        Connect the signals of a view box menu to the appropriate slots.
+
+        Parameters
+        ----------
+        view_box_menu : MultiAxisViewBoxMenu
+            The menu to connect actions to the correct slots.
+        """
+        view_box_menu.sigMouseModeChanged.connect(self.changeMouseMode)
+        view_box_menu.sigXAutoRangeChanged.connect(self.updateXAutoRange)
+        view_box_menu.sigRestoreRanges.connect(self.restoreAxisRanges)
+        view_box_menu.sigSetAutorange.connect(self.setPlotAutoRange)
+
     def updateStackedViews(self):
         """
         Callback for resizing stacked views when the geometry of their top level view changes
@@ -116,7 +139,7 @@ class MultiAxisPlot(PlotItem):
         for view in self.stackedViews:
             view.setGeometry(self.vb.sceneBoundingRect())
 
-    def linkDataToAxis(self, plotDataItem, axisName):
+    def linkDataToAxis(self, plotDataItem: PlotDataItem, axisName: str) -> None:
         """
         Links the input PlotDataItem to the axis with the given name. Raises an exception if that axis does not exist.
         Unlinks the data from any view it was previously linked to.
@@ -144,6 +167,9 @@ class MultiAxisPlot(PlotItem):
             currentView.removeItem(plotDataItem)
             plotDataItem.forgetViewBox()
 
+        if axisToLink.logMode:
+            plotDataItem.setLogMode(False, True)
+
         axisToLink.linkedView().addItem(plotDataItem)
         self.dataItems.append(plotDataItem)
         # Maintain all configurable options set by this plot
@@ -159,14 +185,12 @@ class MultiAxisPlot(PlotItem):
         if self.ctrl.averageGroup.isChecked():
             self.addAvgCurve(plotDataItem)
 
-        if plotDataItem.name():
-            if axisToLink.labelText:
-                # Joins together the labels from the curves for display on their shared axis. The label
-                # text expects html, so this will set it to be, for example, "label 1  &  label 2"
-                axisToLink.setLabel(escape(axisToLink.labelText + ' & ' + plotDataItem.name()),
-                                    color=plotDataItem.color_string)
-            else:
-                axisToLink.setLabel(plotDataItem.name(), color=plotDataItem.color_string)
+        if plotDataItem.name() and not axisToLink.labelText:
+            axisToLink.setLabel(plotDataItem.name(), color=plotDataItem.color_string)
+        elif axisToLink.labelText and plotDataItem.color_string and axisToLink.labelStyle['color'] == '#969696':
+            # The color for the axis was not specified by the user (#969696 is default) so set it appropriately
+            axisToLink.labelStyle['color'] = plotDataItem.color_string
+            axisToLink._updateLabel()
         if self.legend is not None and plotDataItem.name():
             self.legend.addItem(plotDataItem, name=plotDataItem.name())
 
@@ -180,7 +204,8 @@ class MultiAxisPlot(PlotItem):
 
         oldAxis = self.axes[axisName]['item']
         self.layout.removeItem(oldAxis)
-        oldAxis.scene().removeItem(oldAxis)
+        if oldAxis.scene() is not None:
+            oldAxis.scene().removeItem(oldAxis)
         oldAxis.unlinkFromView()
         del self.axes[axisName]
 
@@ -216,6 +241,8 @@ class MultiAxisPlot(PlotItem):
 
         for view in self.stackedViews:
             view.setXRange(minX, maxX, padding=padding)
+        if 'bottom' not in self.axesOriginalRanges:
+            self.axesOriginalRanges['bottom'] = (minX, maxX)
         super(MultiAxisPlot, self).setXRange(minX, maxX, padding=padding)
 
     def setYRange(self, minY, maxY, padding=0, update=True):
@@ -238,11 +265,22 @@ class MultiAxisPlot(PlotItem):
             view.setYRange(minY, maxY, padding=padding)
         super(MultiAxisPlot, self).setYRange(minY, maxY, padding=padding)
 
+    def isAnyXAutoRange(self) -> bool:
+        """ Return true if any view boxes are set to autorange on the x-axis, false otherwise """
+        for view in self.stackedViews:
+            if view.state['autoRange'][0]:
+                return True
+        return False
+
     def disableXAutoRange(self):
         """ Disable x-axis autorange for all views in the plot. """
         for view in self.stackedViews:
             view.enableAutoRange(x=False)
 
+    def getAxes(self) -> List[AxisItem]:
+        """ Returns all axes that have been added to this plot """
+        return [val['item'] for val in self.axes.values()]
+
     def clearAxes(self):
         """
         Cleans up all axis related data from this plot.
@@ -258,13 +296,52 @@ class MultiAxisPlot(PlotItem):
         for oldAxis in allAxes:
             if oldAxis.orientation != 'bottom':   # Currently only multiple y axes are supported
                 self.layout.removeItem(oldAxis)
-                oldAxis.scene().removeItem(oldAxis)
+                if oldAxis.scene() is not None:
+                    oldAxis.scene().removeItem(oldAxis)
                 oldAxis.unlinkFromView()
 
         # Retain the x axis
         bottomAxis = self.axes['bottom']
         self.axes = {'bottom': bottomAxis}
 
+    def restoreAxisRanges(self):
+        """ Restore the min and max range of all axes on the plot to their original values """
+        if len(self.axes) == 0 or len(self.axesOriginalRanges) == 0:
+            return
+
+        # First restore the range for all y-axis items added to this plot
+        for axisName, axisValue in self.axes.items():
+            axisItem = axisValue['item']
+            linkedView = axisItem.linkedView()
+            if linkedView is None or axisItem.orientation not in ('left', 'right') or axisName not in self.axesOriginalRanges:
+                continue
+
+            original_ranges = self.axesOriginalRanges[axisName]
+            if original_ranges[0] is None:  # If set to None, then autorange was enabled
+                linkedView.enableAutoRange(axis=ViewBox.YAxis, enable=True)
+            else:
+                linkedView.setYRange(original_ranges[0], original_ranges[1])
+
+        # Now restore the x-axis range as well if needed
+        if 'bottom' in self.axesOriginalRanges and self.axesOriginalRanges['bottom'][0] is not None:
+            self.setXRange(self.axesOriginalRanges['bottom'][0], self.axesOriginalRanges['bottom'][1])
+        else:
+            self.setPlotAutoRange(x=True)
+
+    def setPlotAutoRange(self, x=None, y=None):
+        """
+        Set autorange for all views on the plot
+        Parameters
+        ----------
+        x: bool, optional
+            Set to true to enable x autorange or false to disable it. Defaults to None which will result in no change
+        y: bool, optional
+            Set to true to enable y autorange or false to disable it. Defaults to None which will result in no change
+        """
+        for stackedView in self.stackedViews:
+            stackedView.enableAutoRange(x=x, y=y)
+        self.getViewBox().enableAutoRange(x=x, y=y)
+
     def clearLayout(self):
         """
         Remove all items from the layout, but leave them intact in the scene so that we can replace them in a new
@@ -299,6 +376,21 @@ class MultiAxisPlot(PlotItem):
             if axis is not None:
                 self.layout.addItem(axis, y, leftOffset)
 
+    def updateGrid(self, *args) -> None:
+        """ Show or hide the grid on a per-axis basis """
+        if is_qt_designer():
+            return
+        # Get the user-set value for the alpha used to draw the grid lines
+        alpha = self.ctrl.gridAlphaSlider.value()
+        x = alpha if self.ctrl.xGridCheck.isChecked() else False
+        y = alpha if self.ctrl.yGridCheck.isChecked() else False
+        all_axes = [val['item'] for val in self.axes.values()]
+        for axis in all_axes:
+            if axis.orientation in ('left', 'right'):
+                axis.setGrid(y)
+            elif axis.orientation in ('top', 'bottom'):
+                axis.setGrid(x)
+
     def handleWheelEvent(self, view, ev, axis):
         """
         A simple slot for propagating a mouse wheel event to all the stacked view boxes (except for the one
@@ -348,3 +440,33 @@ class MultiAxisPlot(PlotItem):
         """
         for stackedView in self.stackedViews:
             stackedView.setLeftButtonAction(mode)
+        self.vb.setLeftButtonAction(mode)
+
+    def updateXAutoRange(self, val):
+        """ Update the autorange values for the x-axis on all view boxes """
+        self.vb.enableAutoRange(ViewBox.XAxis, val)
+        for stackedView in self.stackedViews:
+            stackedView.enableAutoRange(ViewBox.XAxis, val)
+
+    def updateLogMode(self) -> None:
+        """ Toggle log mode on or off for each item in the plot """
+        x = self.ctrl.logXCheck.isChecked()
+        y = self.ctrl.logYCheck.isChecked()
+
+        allAxes = self.getAxes()
+        for axis in allAxes:
+            if axis.orientation in ('bottom', 'top'):
+                axis.setLogMode(x)
+            elif axis.orientation in ('left', 'right'):
+                axis.setLogMode(y)
+
+        for i in self.items:
+            if hasattr(i, 'setLogMode'):
+                i.setLogMode(x, y)
+
+        for i in self.dataItems:
+            if hasattr(i, 'setLogMode'):
+                i.setLogMode(x, y)
+
+        self.enableAutoRange()
+        self.recomputeAverages()
