@@ -1,9 +1,13 @@
-from qtpy.QtDesigner import QExtensionFactory, QPyDesignerTaskMenuExtension
-from qtpy import QtWidgets, QtCore
+import logging
 
+from qtpy.QtDesigner import QExtensionFactory, QPyDesignerTaskMenuExtension
+from qtpy import QtWidgets
+
+from ..utilities import copy_to_clipboard, get_clipboard_text
 from ..widgets.base import PyDMPrimitiveWidget
 
 from ..widgets.rules_editor import RulesEditor
+from ..widgets.designer_settings import BasicSettingsEditor
 from ..widgets.archiver_time_plot_editor import ArchiverTimePlotCurveEditorDialog
 from ..widgets.waveformplot_curve_editor import WaveformPlotCurveEditorDialog
 from ..widgets.timeplot_curve_editor import TimePlotCurveEditorDialog
@@ -11,6 +15,9 @@ from ..widgets.scatterplot_curve_editor import ScatterPlotCurveEditorDialog
 from ..widgets.symbol_editor import SymbolEditor
 
 
+logger = logging.getLogger(__name__)
+
+
 class PyDMExtensionFactory(QExtensionFactory):
     def __init__(self, parent=None):
         super(PyDMExtensionFactory, self).__init__(parent)
@@ -67,6 +74,73 @@ class PyDMExtension(object):
         raise NotImplementedError
 
 
+class BasicSettingsExtension(PyDMExtension):
+    def __init__(self, widget):
+        super(BasicSettingsExtension, self).__init__(widget)
+        self.widget = widget
+        self.edit_settings_action = QtWidgets.QAction(
+            "Py&DM basic settings...", self.widget
+        )
+        self.edit_settings_action.triggered.connect(self.open_dialog)
+
+        if not hasattr(widget, "channel"):
+            self.channel_menu_action = None
+        else:
+            self.channel_menu_action = QtWidgets.QAction(
+                "PyDM C&hannel", self.widget
+            )
+            # self.channel_menu_action.triggered.connect(self.open_channel_menu)
+            self.channel_menu = QtWidgets.QMenu()
+            self.copy_channel_action = self.channel_menu.addAction("")
+            self.copy_channel_action.triggered.connect(self.copy_channel)
+            self.paste_channel_action = self.channel_menu.addAction("")
+            self.paste_channel_action.triggered.connect(self.paste_channel)
+            self.channel_menu.aboutToShow.connect(self.update_action_clipboard_text)
+            edit_channel = self.channel_menu.addAction(
+                "&Edit channel..."
+            )
+            edit_channel.triggered.connect(self.open_dialog)
+            copy_channel_value = self.channel_menu.addAction(
+                "C&opy current value"
+            )
+            copy_channel_value.triggered.connect(self.copy_channel_value)
+            self.channel_menu_action.setMenu(self.channel_menu)
+
+    def update_action_clipboard_text(self):
+        self.copy_channel_action.setText(
+            f"&Copy to clipboard: {self.widget.channel}"
+        )
+        clipboard_text = get_clipboard_text() or ""
+        self.paste_channel_action.setText(
+            f"&Paste from clipboard: {clipboard_text[:100]}"
+        )
+
+    def copy_channel(self, _):
+        channel = self.widget.channel
+        if channel:
+            copy_to_clipboard(channel)
+
+    def copy_channel_value(self, _):
+        value = getattr(self.widget, "value", None)
+        if value:
+            copy_to_clipboard(value)
+
+    def paste_channel(self, _):
+        self.widget.channel = get_clipboard_text() or ""
+        logger.info("Set widget channel to %r", self.widget.channel)
+
+    def open_dialog(self, state):
+        dialog = BasicSettingsEditor(self.widget, parent=None)
+        dialog.exec_()
+
+    def actions(self):
+        actions = [
+            self.edit_settings_action,
+            self.channel_menu_action,
+        ]
+        return [action for action in actions if action is not None]
+
+
 class RulesExtension(PyDMExtension):
     def __init__(self, widget):
         super(RulesExtension, self).__init__(widget)
